﻿using Borepin.Service;
using Prism.Mvvm;
using Prism.Navigation;
using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Input;
using Xamarin.Forms;

namespace Borepin.PageModel
{
    public class HostSelectPageModel : BindableBase
    {
        private INavigationService _NavigationService;
        private BFFHService _BFFHService;

        public HostSelectPageModel(INavigationService navigationService, BFFHService bffhService)
        {
            _NavigationService = navigationService;
            _BFFHService = bffhService;

            UseHostCommand = new Command(UseHostCommandExecuted);
            DetectHostCommand = new Command(DetectHostCommandExecuted);
        }

        private string _Host;
        public string Host
        {
            get => _Host;
            set => SetProperty(ref _Host, value);
        }

        private List<string> _KnownHost_List;
        public List<string> KnownHost_List
        {
            get
            {
                List<string> list = new List<string>();
                foreach(Uri host in _BFFHService.KnownHost)
                {
                    list.Add(host.ToString());
                }
                return list;
            }
        }

        private ICommand _UseHostCommand;
        public ICommand UseHostCommand
        {
            get => _UseHostCommand;
            set => SetProperty(ref _UseHostCommand, value);
        }

        private async void UseHostCommandExecuted()
        {
            UriBuilder builder = new UriBuilder(Host);
            if(builder.Port == 80)
            {
                builder.Port = 59661;
            }

            _BFFHService.Connect(builder.Uri);

            INavigationResult result = await _NavigationService.NavigateAsync($"LoginChoosePage");
            if (!result.Success)
            {
                System.Diagnostics.Debugger.Break();
            }
        }

        private ICommand _DetectHostCommand;
        public ICommand DetectHostCommand
        {
            get => _DetectHostCommand;
            set => SetProperty(ref _DetectHostCommand, value);
        }

        private void DetectHostCommandExecuted()
        {
            // Use Demo Host
            Host = "127.0.0.1";
        }
    }
}
