﻿using System;
using System.Collections.Generic;
using Xamarin.Essentials;
using FabAccessAPI;
using Capnp.Rpc;
using Xamarin.Forms;

namespace Borepin.Service
{
    public class BFFHService
    {
        public Uri SelectedHost;

        private Connection _Connection;

        public List<Uri> KnownHost
        {
            get
            {
                string knownhosts = Preferences.Get("bffh_knownhost", "");

                List<Uri> knownhost_list = new List<Uri>();

                string[] split = knownhosts.Split(',');

                foreach(string host in split)
                {
                    if(host == "")
                    {
                        continue;
                    }

                    Uri host_new = new Uri(host);
                    knownhost_list.Add(host_new);
                }
                return knownhost_list;
            }
        }

        private void KnownHost_Add(Uri address)
        {
            if(KnownHost.Contains(address))
            {
                return;
            }

            string knownhosts = Preferences.Get("bffh_knownhost", "");

            knownhosts += "," + address.ToString();

            Preferences.Set("bffh_knownhost", knownhosts);
        }

        public void Connect()
        {
            if (SelectedHost == null)
            {
                throw new System.Exception("No Host selected");
            }

            Connect(SelectedHost);
        }

        public void Connect(Uri address)
        {
            if(_Connection != null)
            {
                throw new System.Exception("Still connected");
            }

            var rpcClient = new TcpRpcClient();
            rpcClient.Connect(address.Host, address.Port);

            Connection connection_test = new Connection(rpcClient);

            KnownHost_Add(address);

            _Connection = connection_test;
            SelectedHost = address;
        }

        public void Disconnect()
        {
            _Connection.RpcClient?.Dispose();
            _Connection = null;
        }

        public string LoadUsername()
        {
            string username = null;

            Device.BeginInvokeOnMainThread(async () => {
                username = await SecureStorage.GetAsync(string.Format("bffh_username_{0}", SelectedHost.ToString()));
            });

            return username;
        }

        private string LoadPassword()
        {
            return SecureStorage.GetAsync(string.Format("bffh_password_{0}", SelectedHost.ToString())).Result;
        }

        private void AddUsername(string username)
        {
            Device.BeginInvokeOnMainThread(async () => {
                await SecureStorage.SetAsync(string.Format("bffh_username_{0}", SelectedHost.ToString()), username);
            });
        }

        private void AddPasword(string password)
        {
            Device.BeginInvokeOnMainThread(async () => {
                await SecureStorage.SetAsync(string.Format("bffh_password_{0}", SelectedHost.ToString()), password);
            });
        }

        public void Authenticate()
        {
            string username = LoadUsername();
            string password = LoadPassword();

            _Connection.Auth("PLAIN", new Dictionary<string, object> { { "Username", username }, { "Password", password } });
        }

        public void Authenticate(string username, string password)
        {
            _Connection.Auth("PLAIN", new Dictionary<string, object> { { "Username", username }, { "Password", password } });

            AddUsername(username);
            AddPasword(password);
        }
    }
}
